@@ -1,6 +1,6 @@
 /*
  * JBoss, Home of Professional Open Source.
- * Copyright 2012, Red Hat, Inc., and individual contributors
+ * Copyright 2016, Red Hat, Inc., and individual contributors
  * as indicated by the @author tags. See the copyright.txt file in the
  * distribution for a full listing of individual contributors.
  *
@@ -26,7 +26,6 @@ import static org.jboss.jdeparser.JExpr.NULL;
 import static org.jboss.jdeparser.JExpr.THIS;
 import static org.jboss.jdeparser.JExprs.$v;
 import static org.jboss.jdeparser.JTypes.$t;
-import static org.jboss.logging.processor.model.Parameter.ParameterType;
 
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -38,7 +37,7 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.atomic.AtomicBoolean;
-import javax.annotation.processing.Filer;
+import javax.annotation.processing.ProcessingEnvironment;
 
 import org.jboss.jdeparser.JAssignableExpr;
 import org.jboss.jdeparser.JBlock;
@@ -54,13 +53,14 @@ import org.jboss.jdeparser.JType;
 import org.jboss.jdeparser.JVarDeclaration;
 import org.jboss.logging.DelegatingBasicLogger;
 import org.jboss.logging.Logger;
-import org.jboss.logging.Logger.Level;
+import org.jboss.logging.annotations.LoggingClass;
 import org.jboss.logging.annotations.Message.Format;
+import org.jboss.logging.annotations.MessageBundle;
+import org.jboss.logging.annotations.MessageLogger;
 import org.jboss.logging.annotations.Once;
 import org.jboss.logging.annotations.Pos;
 import org.jboss.logging.annotations.Transform;
 import org.jboss.logging.processor.model.MessageInterface;
-import org.jboss.logging.processor.model.MessageInterface.AnnotatedType;
 import org.jboss.logging.processor.model.MessageMethod;
 import org.jboss.logging.processor.model.Parameter;
 import org.jboss.logging.processor.util.ElementHelper;
@@ -85,12 +85,12 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
     /**
      * Creates a new message logger code model.
      *
-     * @param filer            the filer used to create the source file
+     * @param processingEnv    the processing environment
      * @param messageInterface the message interface to implement.
      * @param useLogging31     {@code true} to use logging 3.1, {@code false} to remain compatible with 3.0
      */
-    public MessageLoggerImplementor(final Filer filer, final MessageInterface messageInterface, final boolean useLogging31) {
-        super(filer, messageInterface);
+    public MessageLoggerImplementor(final ProcessingEnvironment processingEnv, final MessageInterface messageInterface, final boolean useLogging31) {
+        super(processingEnv, messageInterface);
         this.useLogging31 = useLogging31;
     }
 
@@ -120,7 +120,6 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
         final JType loggerType = $t(Logger.class);
         // Import the logger type and the level type
         sourceFile._import(loggerType);
-        sourceFile._import(Logger.Level.class);
         final JParamDeclaration constructorParam = constructor.param(JMod.FINAL, loggerType, LOG_FIELD_NAME);
         final JBlock constructorBody = constructor.body();
         final JAssignableExpr logger;
@@ -131,9 +130,14 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
                 constructorBody.callSuper().arg($v(constructorParam));
                 logger = $v("super").field("log");
             } else {
+                sourceFile._import(Logger.Level.class);
                 JVarDeclaration logVar = classDef.field(JMod.PROTECTED | JMod.FINAL, loggerType, LOG_FIELD_NAME);
                 constructorBody.assign(THIS.field(logVar.name()), $v(constructorParam));
                 logger = $v(logVar);
+                // Add static imports for all the log levels
+                for (Logger.Level level : Logger.Level.values()) {
+                    sourceFile.importStatic(Logger.Level.class, level.name());
+                }
                 generateDelegatingLoggerMethods(classDef, logger, fqcn);
             }
         } else {
@@ -141,16 +145,15 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
             constructorBody.assign(THIS.field(logVar.name()), $v(constructorParam));
             logger = $v(logVar);
         }
+        final JCall localeGetter = createLocaleGetter(null, false);
 
         // Process the method descriptors and add to the model before writing.
-        final Set<MessageMethod> messageMethods = new LinkedHashSet<MessageMethod>();
+        final Set<MessageMethod> messageMethods = new LinkedHashSet<>();
         messageMethods.addAll(messageInterface().methods());
         for (MessageInterface messageInterface : messageInterface().extendedInterfaces()) {
-            // Handle logger interface
-            if (messageInterface.getAnnotatedType() == AnnotatedType.NONE) {
-                continue;
+            if (messageInterface.isAnnotatedWith(MessageBundle.class) || messageInterface.isAnnotatedWith(MessageLogger.class)) {
+                messageMethods.addAll(messageInterface.methods());
             }
-            messageMethods.addAll(messageInterface.methods());
         }
         for (MessageMethod messageMethod : messageMethods) {
 
@@ -158,7 +161,7 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
             if (messageMethod.isLoggerMethod()) {
                 createLoggerMethod(messageMethod, classDef, logger);
             } else {
-                createBundleMethod(classDef, messageMethod);
+                createBundleMethod(classDef, localeGetter, messageMethod);
             }
         }
         return classDef;
@@ -243,7 +246,7 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
                     xxx1x.body().add(
                             logVar.call(target)
                                     .arg($v(fqcn))
-                                    .arg(logLevelClass.$v(level))
+                                    .arg($v(level))
                                     .arg(renderThr ? $v(thr) : NULL)
                                     .arg($v(xxx1xFormat))
                                     .arg($v(xxx1xParams))
@@ -261,7 +264,7 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
                         }
                         final JCall xxx2xCaller = logVar.call(target);
                         xxx2xCaller.arg($v(fqcn))
-                                .arg(logLevelClass.$v(level))
+                                .arg($v(level))
                                 .arg(renderThr ? $v(thr) : NULL)
                                 .arg($v(xxx2xFormat));
                         for (int j = 0; j < i; j++) {
@@ -419,7 +422,7 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
 
         // Check for the @Once annotation
         final JBlock body;
-        if (ElementHelper.isAnnotatedWith(messageMethod.reference(), Once.class) && messageMethod.isLoggerMethod()) {
+        if (messageMethod.isAnnotatedWith(Once.class) && messageMethod.isLoggerMethod()) {
             final JType atomicBoolean = $t(AtomicBoolean.class);
             sourceFile._import(atomicBoolean);
             // The variable will be shared with overloaded methods
@@ -433,9 +436,9 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
             }
             body = method.body()._if(
                     logger.call("isEnabled").arg($v(messageMethod.logLevel())).and(
-                    $v(var).call("compareAndSet").arg(JExpr.FALSE).arg(JExpr.TRUE)))
+                            $v(var).call("compareAndSet").arg(JExpr.FALSE).arg(JExpr.TRUE)))
                     .block(Braces.REQUIRED);
-        } else if (!messageMethod.parameters(ParameterType.TRANSFORM).isEmpty()) {
+        } else if (!messageMethod.parametersAnnotatedWith(Transform.class).isEmpty()) {
             body = method.body()._if(logger.call("isEnabled").arg($v(messageMethod.logLevel()))).block(Braces.REQUIRED);
         } else {
             body = method.body();
@@ -443,12 +446,16 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
 
         // Determine which logger method to invoke
         final JCall logCaller = logger.call(messageMethod.loggerMethod());
-        if (messageMethod.parameters(ParameterType.FQCN).isEmpty()) {
+        final Set<Parameter> fqcnParameters = messageMethod.parametersAnnotatedWith(LoggingClass.class);
+        if (fqcnParameters.isEmpty()) {
             logCaller.arg($v(FQCN_FIELD_NAME));
         } else {
-            logCaller.arg($v(params.get(messageMethod.parameters(ParameterType.FQCN).iterator().next())).call("getName"));
+            logCaller.arg($v(params.get(fqcnParameters.iterator().next())).call("getName"));
         }
-        logCaller.arg($v(messageMethod.logLevel()));
+        // Use static imports for the levels
+        final String levelName = messageMethod.logLevel();
+        sourceFile.importStatic(Logger.Level.class, levelName);
+        logCaller.arg($v(levelName));
 
 
         final MessageMethod.Message message = messageMethod.message();
@@ -479,28 +486,20 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
                 final Parameter param = entry.getKey();
                 final String formatterClass = param.formatterClass();
                 final JParamDeclaration var = entry.getValue();
-                switch (param.parameterType()) {
-                    case FORMAT:
-                        if (formatterClass == null) {
-                            if (param.isArray() || param.isVarArgs()) {
-                                args.add($t(Arrays.class).call("toString").arg($v(var)));
-                            } else {
-                                args.add($v(var));
-                            }
-                        } else {
-                            args.add($t(formatterClass)._new().arg($v(var)));
-                        }
-                        break;
-                    case TRANSFORM:
+
+                boolean added = false;
+                if (param.isFormatParameter()) {
+                    if (param.isAnnotatedWith(Transform.class)) {
                         final JAssignableExpr transformVar = createTransformVar(parameterNames, body, param, $v(var));
                         if (formatterClass == null) {
                             args.add(transformVar);
                         } else {
                             args.add($t(formatterClass)._new().arg(transformVar));
                         }
-                        break;
-                    case POS:
-                        final Pos pos = param.pos();
+                        added = true;
+                    }
+                    if (param.isAnnotatedWith(Pos.class)) {
+                        final Pos pos = param.getAnnotation(Pos.class);
                         final int[] positions = pos.value();
                         final Transform[] transform = pos.transform();
                         for (int i = 0; i < positions.length; i++) {
@@ -520,7 +519,20 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
                                 }
                             }
                         }
-                        break;
+                        added = true;
+                    }
+
+                    if (!added) {
+                        if (formatterClass == null) {
+                            if (param.isArray() || param.isVarArgs()) {
+                                args.add($t(Arrays.class).call("toString").arg($v(var)));
+                            } else {
+                                args.add($v(var));
+                            }
+                        } else {
+                            args.add($t(formatterClass)._new().arg($v(var)));
+                        }
+                    }
                 }
             }
             for (JExpr arg : args) {
@@ -533,7 +545,7 @@ final class MessageLoggerImplementor extends ImplementationClassModel {
     private Map<Parameter, JParamDeclaration> createParameters(final MessageMethod messageMethod, final JMethodDef method) {
         final Map<Parameter, JParamDeclaration> result = new LinkedHashMap<>();
         // Create the parameters
-        for (Parameter param : messageMethod.parameters(ParameterType.ANY)) {
+        for (Parameter param : messageMethod.parameters()) {
             final JParamDeclaration var = addMethodParameter(method, param);
             result.put(param, var);
         }
